package com.bull.cimero.pluginEditor.editors.model.BCModel;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.editors.model.Element;

/**
 * The routeur model class.
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class JdbcBC extends BindingComponentModel  {

	
	private static final long serialVersionUID = 1L;
	public static final String NBSUBQUERIES = "number of subqueries";
	public static final String JDBCBC = "myJdbcBC";
	
	private static String MS_SQL_SERVER = "MS_SQL";
	private static String MySQL_SERVER = "MySQL";
	private static String Oracle_SERVER = "Oracle";
	private static String Sybase_SERVER = "Sybase";
	private static String DB2_SERVER = "DB2";
	private static String Postgre_SQL_SERVER = "Postgre_SQL";
	private static String JDBC_SERVER = "JDBC";
	private static String[] supportedDatabases = {MS_SQL_SERVER, MySQL_SERVER, Oracle_SERVER, 
		Sybase_SERVER, DB2_SERVER, Postgre_SQL_SERVER, JDBC_SERVER};
	public static final String DATABASE_TYPE = "DATABASE_TYPE".intern();

	
	public static final String CONN_URL = "CONN_URL";
	public static final String CONN_DRIVER = "CONN_DRIVER";
	public static final String CONN_USER =  "CONN_USER"; 
	public static final String CONN_PASSWORD =  "CONN_PASSWORD";
	public static final String JDBC_BC_PERIOD =  "JDBC_BC_PERIOD";
	
	public static final String JDBC_BC_KEY = "JDBC_BC_KEY";
	public static final String JDBC_BC_INITIAL_KEY_VALUE = "JDBC_BC_INITIAL_KEY_VALUE";
	public static final String JDBC_BC_COMMPOINTID = "JDBC_BC_COMMPOINTID";
	public static final String JDBC_BC_ROWS_IN_MESSAGE = "JDBC_BC_ROWS_IN_MESSAGE";   
       
        
	public static final String ROW_NAME_AS_ATTRIBUTE="ROW_NAME_AS_ATTRIBUTE";
	public static final String COL_NAME_AS_ATTRIBUTE="COL_NAME_AS_ATTRIBUTE";
	public static final String VALUE_AS_ATTRIBUTE="VALUE_AS_ATTRIBUTE";
    
	public static final String MAINQUERY_NAME = "MainQueryName";
	public static final String MAINQUERY_SQL_VALUE = "MainQuerySQLValue";
	
	private static final String NAMEDEST = "destination service";
	
	private List < Query > maLstQuery = null;
	private static final String ID = "JDBCBC";

	/**.
	 * The default contructor
	 */
	public JdbcBC() {
		super(JDBCBC);
		//add name properties
		setProperty(NAME, JDBCBC);
		setProperty(NBSUBQUERIES, "0");
		setProperty(MAINQUERY_NAME, "MainQuery");
		setProperty(MAINQUERY_SQL_VALUE, "");
		
		setProperty(CONN_URL,"");
		setProperty(CONN_DRIVER,"");
		setProperty(CONN_USER,""); 
		setProperty(CONN_PASSWORD,"");
		setProperty(JDBC_BC_PERIOD,"");
		
		setProperty(JDBC_BC_KEY,"");
		setProperty(JDBC_BC_INITIAL_KEY_VALUE,"");
		setProperty(JDBC_BC_COMMPOINTID,"");
		setProperty(JDBC_BC_ROWS_IN_MESSAGE ,""); 
		
		setProperty(ROW_NAME_AS_ATTRIBUTE, 0);
		setProperty(COL_NAME_AS_ATTRIBUTE, 0);
		setProperty(VALUE_AS_ATTRIBUTE, 0);
		setProperty(NAMEDEST, "");
		setProperty(DATABASE_TYPE, 0);
	}

	/**
	 * nbRules getter function.
	 * @return int
	 */
	public final int getNbSubQueries() {
		return Integer.valueOf((String) getProperty(NBSUBQUERIES));
	}

	/**
	 * nbRules setters functions.
	 * @param numberRules : the numbers of rules
	 */
	public final void getNbSubQueries(final int numberSubQueries) {
		setProperty(NBSUBQUERIES, String.valueOf(numberSubQueries));
	}
	
	public boolean isRowNamesAsAttribute(){
		return ((Integer) getProperty(ROW_NAME_AS_ATTRIBUTE) == 0);
	}
	
	public boolean isColNamesAsAttribute(){
		return ((Integer) getProperty(COL_NAME_AS_ATTRIBUTE) == 0);
	}
	
	public boolean isValueAsAttribute(){
		return ((Integer) getProperty(VALUE_AS_ATTRIBUTE) == 0);
	}

	/**
	 * The getter LstRules.
	 * @return List
	 */
	public final List < Query > getMaLstQuery() {
		/*
		if (maLstQuery == null) {
			maLstQuery = new ArrayList < Query >();
			addEltLst(new Query(1));
		}
		*/
		return maLstQuery;
	}
	/**
	 * This function is used to add an Rule element in the list.
	 * @param param : the Rule element
	 */
	public final void addEltLst(final Query param) {
		if (maLstQuery == null) {
			maLstQuery = new ArrayList < Query >();
			//addEltLst(new Query(1));
		}
		getMaLstQuery().add(param);
	}

	/**
	 * This function is used to remove the last rule
	 * of this routeur.
	 */
	public final void removeLastElement() {
		if (getMaLstQuery().size() > 0){
			getMaLstQuery().remove(getMaLstQuery().size() - 1);
		}
	}
	
	public final void clearList() {
		getMaLstQuery().clear();
	}

	/**.
	 * This function is used by the validator. It is used to determine
	 * if this class is validate or not
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		boolean returnResult = true;
		String msg = "";
		if (getName().compareTo("") == 0) {
			msg += "\nThe name routeur cannot be null";
			returnResult = false;
		}
		for (int i = 0; (getMaLstQuery() != null && i < getMaLstQuery().size()); i++) {
			Query tmp = getMaLstQuery().get(i);
			String res = tmp.isValidate();
			if (res.compareTo("") != 0) {
				returnResult = false;
				msg += res;
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * set a property value.
	 * @param id : the id
	 * @param value : the value of this id
	 */
	public final void setPropertyValue(final Object id, Object value) {
		boolean find = false;
		if (id == NBSUBQUERIES) {
			//we compare the actual value with the last value
			int oldValue = getNbSubQueries();
			int newValue = 0;
			//parse the Object in integer
			try {
				newValue =  Integer.valueOf((String) value);
				if ((newValue < 0) || (newValue > 50)) {
					throw new Exception();
				}
				
			} catch (Exception e) {
				SpagicLogger.error("The rule number must be a integer "
						+ "with a value between 0 and 50", e);
				newValue = oldValue;
				value = String.valueOf(newValue);
			}
			if (oldValue != newValue) {
				if (newValue == 0){
					clearList();
				}
				if (oldValue < newValue) {
					//we add new Rule
					for (int i = 0; i < newValue - oldValue; i++) {
						addEltLst(new Query(oldValue + i + 1));
					}
				} else {
					//we remove the last Rule
					for (int i = 0; i < oldValue - newValue; i++) {
						removeLastElement();
					}
				}
			}
		} else {
			//we send this id and value to all rules
			for (int i = 0; (getMaLstQuery() != null && i < getMaLstQuery().size()); i++) {
				if (getMaLstQuery().get(i).setPropertyValue(id, value)) {
					find = true;
					//tell the parts, property change --> refresh
					fireStructureChange(NAME, this);
				}
			}
		}
		if (!find) {
			super.setPropertyValue(id, value);
		}
	}

	/**
	 * get a property value.
	 * @param id : the id.
	 * @return Object
	 */
	public final Object getPropertyValue(final Object id) {
		if (id.equals(NBSUBQUERIES)) {
			return (String) getProperty(NBSUBQUERIES);
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}	
		}
	
		//we send this object id to all rules
		for (int i = 0; (getMaLstQuery() != null && i < getMaLstQuery().size()); i++) {
			Object result = getMaLstQuery().get(i).getPropertyValue(id);
			if (result != null) {
				return result;
			}
		}
		
	
		return super.getPropertyValue(id);
	}

	/**
	 * reset all property value.
	 * @param id : the id of the element
	 * @see org.eclipse.ui.views.properties.
	 * IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public final void resetPropertyValue(final Object id) {
		//do nothing
	}

	/**
	 * method to create descriptors.
	 * @see com.bull.cimero.pluginEditor.editors.model
	 * .GeneriqueModel#createDescriptor()
	 */
	protected final void createDescriptor() {
		int indiceMain = 0;
		//add name property
		IPropertyDescriptor descr;
		descr = new TextPropertyDescriptor(NAME, indiceMain++ + " - name");
		((TextPropertyDescriptor) descr).setCategory("JDBCBC");
		addDescriptor(descr);
		
		if (!isInput()) {
			
			 String [] lst = getLstObjectDrawWithInputConnection();
			 descr = new ComboBoxPropertyDescriptor(NAMEDEST,
					 indiceMain++ + " - destination name :", lst);
			 ((ComboBoxPropertyDescriptor) descr).setCategory("JDBCBC");
			 
			 addDescriptor(descr);
		 }
		
		int indice = 1;
		
		// -- Main query
		descr = new TextPropertyDescriptor(MAINQUERY_NAME, getString(indice++) + " - Name");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(MAINQUERY_SQL_VALUE, getString(indice++) + " - SQL");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);

		
		descr = new TextPropertyDescriptor(CONN_DRIVER, getString(indice++) +" - Connection Driver");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(CONN_URL, getString(indice++) +" - Connection URL");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(CONN_USER, getString(indice++) +" - Connection User");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
	
		descr = new TextPropertyDescriptor(CONN_PASSWORD, getString(indice++) +" - Connection Password");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
	

		descr = new TextPropertyDescriptor(JDBC_BC_PERIOD, getString(indice++) +" - Period in millisecond");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(JDBC_BC_KEY, getString(indice++) +" - Key");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(JDBC_BC_INITIAL_KEY_VALUE, getString(indice++) +" - Initial Key Value");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		
		descr = new TextPropertyDescriptor(JDBC_BC_COMMPOINTID, getString(indice++) +" - Comm Point ID");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(JDBC_BC_ROWS_IN_MESSAGE, getString(indice++) +" - rows for each message ( leave blank for all )");
		((TextPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		

		
		
		descr = new ComboBoxPropertyDescriptor(
				ROW_NAME_AS_ATTRIBUTE, getString(indice++) +" - Row Name as Attribute", new String[]{"TRUE", "FALSE"});
		((ComboBoxPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new ComboBoxPropertyDescriptor(
				COL_NAME_AS_ATTRIBUTE, getString(indice++) + " - Column Name as Attribute", new String[]{"TRUE", "FALSE"});
		((ComboBoxPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new ComboBoxPropertyDescriptor(
				VALUE_AS_ATTRIBUTE, getString(indice++) + " - Value as Attribute", new String[]{"TRUE", "FALSE"});
		((ComboBoxPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		descr = new TextPropertyDescriptor(NBSUBQUERIES, indiceMain++ + " - number of Subqueries : ");
		((TextPropertyDescriptor) descr).setCategory("JDBCBC");
		addDescriptor(descr);
		
		descr = new ComboBoxPropertyDescriptor(
				DATABASE_TYPE, getString(indice++) + " - Database Type", 
				supportedDatabases);
		((ComboBoxPropertyDescriptor) descr).setCategory("Main Query");
		addDescriptor(descr);
		
		
		
		//add rule descriptor
		int numRules = (getMaLstQuery() != null ? getMaLstQuery().size() : 0);
		for (int i = 0; i < numRules; i++) {
			IPropertyDescriptor[] ruleDescr = getMaLstQuery().get(i)
			.createPropertyDescriptor();
			for (int cptDescr = 0; cptDescr < ruleDescr.length; cptDescr++) {
				//descr[indiceDescr++] = ruleDescr[cptDescr];
				addDescriptor(ruleDescr[cptDescr]);
			}
		}
		
	}

	private String getString(int indice){
		if (indice >= 10)
			return String.valueOf(indice);
		else
			return "0" + String.valueOf(indice);
	}
	

	

	

	

	/**
	 * This class contains all the property needed to create a routeur rule.
	 * @author Cedric MOUILLERON
	 * @author Pierre NOTEL
	 */
	public class Query extends Element {

		
		/** serial uid. */
		private static final long serialVersionUID = 1L;
		
		/** default name. */
		private static final String DEFAULTNAME = "SubQuery ";
		/** default name. */
		private static final String DEFAULSQLNAME = "SQL ";
		
		public static final String QUERY_NAME = "Name";
		/** id of property value. */
		public static final String QUERY_VALUE = "SqlValue";
		
		/** The number of this rule. */
		private int numQuery;
		

		/**
		 * The propertyName getter function.
		 * @return String
		 */
		public String getQueryName() {
			return (String) getProperty(numQuery + QUERY_NAME);
		}

		

		/**
		 * The propertyName setter function.
		 * @param propName : the name of the property
		 */
		public final void setPropertyName(final int propName) {
			setProperty(numQuery + QUERY_NAME, propName);
		}

		/**
		 * The valueRule getter function.
		 * @return String
		 */
		public final String getQueryValue() {
			return (String) getProperty(numQuery + QUERY_VALUE);
		}
		
		

		/**
		 * The valueRule setter function.
		 * @param value : the value of the rule
		 */
		public final void setValueRule(final String value) {
			setProperty(numQuery + QUERY_VALUE, value);
		}

		
		public Query(final int numQueryParam) {
			this.numQuery = numQueryParam;
			System.out.println(" --> Setting property ["+numQuery + QUERY_NAME+"]");
			setProperty(numQuery + QUERY_NAME, "");
			setProperty(numQuery + QUERY_VALUE, "");
		}

		

		/**
		 * This function is used to create the property descriptor of this
		 * Rule.
		 * @return IPropertyDescriptor[]
		 */
		public final IPropertyDescriptor[] createPropertyDescriptor() {
			String prefixe = DEFAULTNAME + numQuery;
			PropertyDescriptor[] arrPropertyDescriptor = new PropertyDescriptor[2];
			
			arrPropertyDescriptor[0] = new TextPropertyDescriptor(numQuery + QUERY_NAME, "01 - Name");
			arrPropertyDescriptor[0].setCategory(prefixe);
			arrPropertyDescriptor[1] = new TextPropertyDescriptor(numQuery + QUERY_VALUE,"02 - Sql");
			arrPropertyDescriptor[1].setCategory(prefixe);
			return arrPropertyDescriptor;
		}

		

		/**
		 *@param id : the id of the property
		 *@return Object
		 */
		public final Object getPropertyValue(final Object id) {
			System.out.println("-----> Get Property Value ["+id+"");
			return getProperty((String)id);
		}

		/**
		 * This function is used to set the property value.
		 * @param id : the id of the property
		 * @param value : the new value of the property
		 * @return true if the property was finded.
		 */
		public final boolean setPropertyValue(final Object id,
				final Object value) {
			System.out.println("-----> Set Property Value ["+id+"");
			setProperty((String)id, value);
			return true;
		}
		
		public String isValidate(){
			return "";
		}
	}

	/**
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
	
	public String getDatabaseType() {
		int intOutgoingEndianness = ((Integer)getProperty(DATABASE_TYPE)).intValue();
		return supportedDatabases[intOutgoingEndianness];
	}
}
