package com.bull.cimero.pluginEditor.editors.model.BCModel;

import org.spagic.utils.SpagicLogger;

import com.bull.cimero.pluginEditor.utils.ressources.FileUtils;

/**
 * This class is the mail model.
 * It contain the properties of a mail serveur
 * @author Cedric MOUILLERON
 * @author Pierre NOTEL
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class Mail extends BindingComponentModel {

	/**The id of this element.*/
	private static final String ID = "Mail";
	//
	//constant
	//
	/**The default Mail name.*/
	private static final String DEFAULTNAME = "myMailServer";
	/**The default communication port.*/
	private static final String DEFAULTCOMMUNICATIONPORT = "25";
	/** The id properties. */
	public static final String HOSTNAME = "serveurName";
	//
	// mode IN
	//
	/**The id of the communication port.*/
	public static final String PORTCOMM = "portCommunication";
	/**The login id.*/
	public static final String LOGIN = "login";
	/**The password id.*/
	public static final String PASSWORD	= "password";
	/**The inbox id.*/
	public static final String INBOX = "inbox";
	/**The destination service id.*/
	private static final String NAMEDEST = "destination service";
	//
	//mode out
	//
	/**The mail to id.*/
	public static final String MAILTO = "mailTo";
	/**The mail from id.*/
	public static final String MAILFROM = "mailFrom";
	/**The mail subject id.*/
	public static final String MAILSUBJECT = "subject";
	/**The mail body id.*/
	public static final String MAILBODY = "mailBody";
	/**The mail attachements id.*/
	public static final String ATTACHEMENTS = "attachement";
	/** serialVersionUID. */
	private static final long serialVersionUID = 1L;

	/**
	 * The default contructor.
	 */
	public Mail() {
		super(DEFAULTNAME);
		if (setInput(false)) {
			setOutput(true);
		}
		//set all the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(HOSTNAME, "");
		setProperty(LOGIN, "");
		setProperty(PORTCOMM, DEFAULTCOMMUNICATIONPORT);
		setProperty(PASSWORD, "");
		setProperty(INBOX, "");
		setProperty(MAILTO, "");
		setProperty(MAILFROM, "");
		setProperty(MAILSUBJECT, "");
		setProperty(MAILBODY, "");
		setProperty(ATTACHEMENTS, "");
		setProperty(NAMEDEST, "");
		setProperty(INOUT, "");
	}

	/**
	 * getter attachement function.
	 * @return String
	 */
	public final String getAttachements() {
		//return attachements;
		return (String) getProperty(ATTACHEMENTS);
	}

	/**
	 * getter communication port function.
	 * @return int
	 */
	public final int getCommunicationPort() {
		//return communicationPort;
		return Integer.parseInt((String) getProperty(PORTCOMM));
	}

	/**
	 * getter hostnameAdress function.
	 * @return String
	 */
	public final String getHostNameAdress() {
		//return hostNameAdress;
		return (String) getProperty(HOSTNAME);
	}

	/**
	 * getter inbox function.
	 * @return String
	 */
	public final String getInbox() {
		//return inbox;
		return (String) getProperty(INBOX);
	}

	/**
	 * getter login function.
	 * @return String
	 */
	public final String getLogin() {
		//return login;
		return (String) getProperty(LOGIN);
	}

	/**
	 * getter MailBody function.
	 * @return String
	 */
	public final String getMailBody() {
//		return mailBody;
		return (String) getProperty(MAILBODY);
	}

	/**
	 * getter Mail body function.
	 * @return String
	 */
	public final String getMailFrom() {
//		return mailFrom;
		return (String) getProperty(MAILFROM);
	}

	/**
	 * getter Mail subject function.
	 * @return String
	 */
	public final String getMailSubject() {
//		return mailSubject;
		return (String) getProperty(MAILSUBJECT);
	}

	/**
	 * getter Mail to function. The destinataires are separated by ;
	 * @return String
	 */
	public final String getMailTo() {
//		return mailTo;
		return (String) getProperty(MAILTO);
	}

	/**
	 *getter password server function.
	 * @return String
	 */
	public final String getPassword() {
//		return password;
		return (String) getProperty(PASSWORD);
	}

	/**
	 * This function is used to validate this class. return true
	 * if all the variable are specifies.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		String msg = "";
		boolean returnResult = true;
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the element name cannot be null";
			returnResult = false;
		}
		if (getHostNameAdress().compareTo("") == 0) {
			msg += "\n2 - HostName => the server adress is required !";
			returnResult = false;
		}
		if (getCommunicationPort() <= 0) {
			msg += "\n4 - CommunicationPort => the port must"
				+ " to be superior than 0";
			returnResult = false;
		}
		if (isOutput()) {
			//mode IN
			if (getLogin().compareTo("") == 0) {
				msg += "\n5 - login => the login to the server is required";
				returnResult = false;
			}
			if (getPassword().compareTo("") == 0) {
				msg += "\n6 - password => "
					+ "the password to the server is required";
				returnResult = false;
			}
			if (getNameFirstDestination().compareTo("") == 0) {
				msg += "\n8 - destination => "
					+ "the destination service cannot be null";
				returnResult = false;
			}
		} else {
			//mode OUT
			//test of the attachemnts
			if (getAttachements().compareTo("") != 0) {
				//attachements is not null
				//split of string
				String[] tabSplit = getAttachements().split(",");
				for (int i = 0; i < tabSplit.length; i++) {
					if (!FileUtils.getDefault().isFile((tabSplit[i].trim()))) {
						msg += "\n9 - Attachement file path => file"
							+ "doesn't exist :" + tabSplit[i].trim();
						returnResult = false;
					}
				}
			}
		}
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + "\n" + msg;
			if (showLog) {
				SpagicLogger.warning(msg);
			}
		}
		return returnResult;
	}

	/**
	 * Create the descriptor of this function.
	 */
	protected final void createDescriptor() {
		int indice = 1;
		addTextDescriptor(NAME, indice++ + " - name : ");
		addTextDescriptor(HOSTNAME, indice++ + " - hostName : ");
		addComboBoxDescriptor(INOUT, indice++ + " - mode",
				new String[]{"IN", "OUT"});
		addTextDescriptor(PORTCOMM, indice++ + " - communication port : ");
		//if this element has an input connection. So it is in out mode MEP
	    if (isInput()) {
	    	addTextDescriptor(MAILTO, indice++ + " - mail To :");
	    	addTextDescriptor(MAILFROM, indice++ + " - mail From :");
	    	addTextDescriptor(MAILSUBJECT, indice++
	    			+ " - subject (optional) :");
	    	addTextDescriptor(MAILBODY, indice++ + " - body :");
	    	addTextDescriptor(ATTACHEMENTS, indice + " - attachement(s) :");
	    } else {
	    	addTextDescriptor(LOGIN, indice++ + " - login :");
	    	addTextDescriptor(PASSWORD, indice++ + " - password :");
	    	addTextDescriptor(INBOX, indice++ + " - inbox (optional) :");
	    	String [] lst = getLstObjectDrawWithInputConnection();
	    	addComboBoxDescriptor(NAMEDEST, indice++
	    			+ " - destination name :", lst);
	    }
	}

	/**
	 * Abstract method. Permit to set the property value.
	 * @param id : the id of the property
	 * @param value : the value of this property
	 */
	public final void setPropertyValue(final Object id, Object value) {
		if (id == INOUT) {
			if (isInput()) {
				if (setInput(false)) {
					setOutput(true);
				}
			} else {
				if (setOutput(false)) {
					setInput(true);
				}
			}
		} else if (id == PORTCOMM) {
			String lastValue = (String) getProperty((String) id);
			try {
				Integer.parseInt((String) value);
			} catch (Exception e) {
				SpagicLogger.error("Error in communication port format!"
						+ "The last value will be restored", e);
				value = lastValue;
			}
		} else if (ATTACHEMENTS.equals(id)) {
			//split of string
			String[] tabSplit = ((String) value).trim().split(",");
			String res = "";
			for (int i = 0; i < tabSplit.length; i++) {
				res += tabSplit[i].trim();
				if (i != tabSplit.length - 1) {
					res += ",";
				}
			}
			value = (String) res;
		} else if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when create connection",
						new Exception());
			}
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * Return the value of a property.
	 * @param id : the id of the property
	 * @return Object
	 */
	public final Object getPropertyValue(final Object id) {
		if (INOUT.equals(id)) {
			if (!isInput()) {
				return 0;
			} else {
				return 1;
			}
		} else if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
		} else if (getProperty((String) id) == null) {
			//we send a log to say that the id is not found
			SpagicLogger.warning("the id (" + id + ") is not known !",
					new Exception());
		} else {
			return super.getPropertyValue(id);
		}
		return null;
	}

	/**
	 * reset the value of the property.
	 * @param id : the id property
	 */
	public void resetPropertyValue(final Object id) {

	}

	/**
	 * get the id.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}

	/**
	 * set the port.
	 * @param communicationPortParam The communicationPort to set.
	 */
	public final void setCommunicationPort(final int communicationPortParam) {
		setProperty(PORTCOMM, String.valueOf(communicationPortParam));
	}

	/**
	 * set the hostname.
	 * @param hostNameAdressParam The hostNameAdress to set.
	 */
	public final void setHostNameAdress(final String hostNameAdressParam) {
		setProperty(HOSTNAME, hostNameAdressParam);
	}

	/**
	 * set the inbox.
	 * @param inboxParam The inbox to set.
	 */
	public final void setInbox(final String inboxParam) {
		setProperty(INBOX, inboxParam);
	}

	/**
	 * set the login.
	 * @param loginParam The login to set.
	 */
	public final void setLogin(final String loginParam) {
		setProperty(LOGIN, loginParam);
	}

	/**
	 * set the password.
	 * @param passwordParam The password to set.
	 */
	public final void setPassword(final String passwordParam) {
		setProperty(PASSWORD, passwordParam);
	}
}
