package it.eng.components.cimero;

import java.util.HashMap;
import java.util.Map;

import org.spagic.bpmn.conversion.Spagic1ToSpagic2Activator;
import org.spagic.utils.SpagicLogger;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import com.bull.cimero.pluginEditor.editors.model.SEModel.ServiceEngineModel;
import com.bull.cimero.pluginEditor.utils.ressources.Ressources;

/**
 * The Generic Transformer
 * @author Andrea Zoppello (andrea.zoppello@eng.it)
 */
public class HL7Component extends ServiceEngineModel {

	/**serialization number.*/
	private static final long serialVersionUID = 1L;
	/**The xsl transformeur path.*/
	// private static final String GROOVY_PATH = "Groovy File Path";
	/**The name of the destination service property.*/
	private static final String NAMEDEST = "destination service";
	/**The name of the mep mode property.*/
	private static final String INOUTMEP = "in/out MEP";
	/**The default transformer name.*/
	private static final String DEFAULTNAME = "myHL7";

	/**the id of this element.*/
	private static final String ID = "HL7COMPONENT";
	
	private static final  String transformationName = "transformationName" ;
	private static final  String transformationType = "transformationType";
	private static final  String transformationClass = "transformationClass" ;
	

	private Map<String, Integer> hl7TransformationNames = null;
	private Map<Integer,String > hl7TransformationPositions = null;
	private Map<String,String > hl7Name2Classes = null;
	
	/**
	 * The default constructor.
	 */
	public HL7Component() {
		super(DEFAULTNAME, 1 , DEFAULTOUTIN);
		//add the properties
		setProperty(NAME, DEFAULTNAME);
		setProperty(NAMEDEST, "");
		setProperty(INOUTMEP, 0);
		
		
	}
	
	public void configureCustomComponent(){
		Ressources ressource = new Ressources();
		String fullPath = Spagic1ToSpagic2Activator.getRessourcePath("/ressources/CustomComponents.xml");
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/COMPONENTS/COMPONENT");
			Node node = null;
			NamedNodeMap attributes = null;
			String tmpId  = null;
		
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				
				tmpId = attributes.getNamedItem("id").getNodeValue();
				if (tmpId.equalsIgnoreCase(getCustomComponentId())){
					setTransformationType(attributes.getNamedItem("transformationType").getNodeValue());
				}
			}
		}
		initHl7MapResources();
	
	}
	public void initHl7MapResources(){
		this.hl7TransformationNames = new HashMap<String, Integer>();
		this.hl7TransformationPositions = new HashMap<Integer, String>();
		this.hl7Name2Classes = new HashMap<String, String>();
	
		Ressources ressource = new Ressources();
		String fullPath = Spagic1ToSpagic2Activator.getRessourcePath("/ressources/hl7map.xml");
		String first = "";
		if (fullPath != null) {
		
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
	
			ressource.setDocument(fullPath);
		
		
			NodeList nodeList = ressource.getXPathNodeSet("/TRANSFORMATIONS/TRANSFORMATION");
			Node node = null;
			NamedNodeMap attributes = null;
		
			String currentEndpointType = getTransformationType();
			String typeOfCurrentnode = null;
			for (int i = 0; i < nodeList.getLength(); i++) {
			
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				typeOfCurrentnode = attributes.getNamedItem("type").getNodeValue();
				if (typeOfCurrentnode.equalsIgnoreCase(currentEndpointType)){
					hl7TransformationNames.put(attributes.getNamedItem("name").getNodeValue(), i);
					hl7Name2Classes.put(attributes.getNamedItem("name").getNodeValue(),attributes.getNamedItem("class").getNodeValue());
					if (i == 0)
						first = attributes.getNamedItem("name").getNodeValue();
					hl7TransformationPositions.put(i, attributes.getNamedItem("name").getNodeValue());
				}
			}
		}
		setProperty(transformationName, first);
}

	
	public final String getTransformationName() {
		return (String) getProperty(transformationName);
	}
	
	public final String getTransformationClass() {
		return hl7Name2Classes.get(getTransformationName());
	}
	
	

	
	
	
	public final String getTransformationType() {
		return (String) getProperty(transformationType);
	}
	

	public final void setTransformationType(final String param) {
		setProperty(transformationType, param);
	}
	

	/**
	 * inoutMep getter function.
	 * @return true if this component is an inoutmep

	public final boolean isInOutMep() {
		return ((Integer) getProperty(INOUTMEP) == 0);
	}
		 */

	/**
	 * path setter function.
	 * @param inOutMepParam : in or out
	 */
	public final void setInOutMep(final boolean inOutMepParam) {
		if (inOutMepParam) {
			setProperty(INOUTMEP, 0);
		} else {
			setProperty(INOUTMEP, 1);
		}
	}

	/**
	 * The validate function. It is used to determine if the instance class
	 * is valide or not.
	 * @param showLog : true if the log must to be show.
	 * @return boolean
	 */
	public final boolean isValide(final boolean showLog) {
		/*
		String msg = "";
		boolean returnResult = true;
		//name verification
		if (getName().compareTo("") == 0) {
			msg += "\n1 - Name => the transformer name cannot be null";
			returnResult = false;
		}
		//xsl path verification
		if (getGroovyPath().compareTo("") == 0) {
			msg += "\n2 - Groovy Path => set the Groovy File Path path";
			returnResult = false;
		} 
		
		if (!returnResult) {
			msg = msg.substring(1);
			msg = "ELEMENT NAME : " + getName() + " \n" + msg;
			if (showLog) {
				CimeroLog.warningLog(msg);
			}
		}
		
		return returnResult;
		*/
		return true;
	}

	/**
	 * IPropertySource abstract method.
	 * @param value : the value of the property
	 * @param id : the id of the property
	 */
	public final void setPropertyValue(final Object id, final Object value) {
		if (id == NAME){
			System.out.println(id + "==> ["+value+"]");
		}
		if (id == NAMEDEST) {
			//we retreive the destination name
			int selecNum = Integer.parseInt(value.toString());
			//retreive the list af all the objects name
			String [] lstStr = getLstObjectDrawWithInputConnection();
			//call the function to create a connection beetween
			//this object and the selected object
			if (!refreshDestinationConnection(lstStr[selecNum])) {
				SpagicLogger.error("problem when created connection",
						new Exception());
			}
		}
		if (id == transformationName){
			int intValue = ((Integer)value).intValue();
			setProperty(transformationName, hl7TransformationPositions.get(intValue));
			return;
		}
		super.setPropertyValue(id, value);
	}

	/**
	 * IPropertySource abstract method.
	 * @return Object
	 * @param id : the id of the property
	 */
	public final Object getPropertyValue(final Object id) {
		if (NAMEDEST.equals(id)) {
			//we retreive the name of the destination object
			String destinationName = getNameFirstDestination();
			//return the num of the name  of the object
			String [] lstStr = getLstObjectDrawWithInputConnection();
			for (int i = 0; i < lstStr.length; i++) {
				if (lstStr[i].compareTo(destinationName) == 0) {
					return i;
				}
			}
			return 0;
		} else if (transformationName.equals(id)){
			String transfName = (String)getProperty(transformationName);
			if (transfName == null || transfName.trim().length() == 0){
				return 0;
			}else{
				return hl7TransformationNames.get(transfName);
			}
		}else{
			return super.getProperty((String) id);
		}
	}

	/**
	 * IPropertySource abstract method.
	 * @param id : the id of the property
	 */
	public void resetPropertyValue(final Object id) {
		// do nothing
	}

	/**
	 * Abstract method of GeneriqueCimeroModel. It is used to
	 * create the descriptor
	 */
	protected  void createDescriptor() {
		int indice = 1;
		String [] lstStr = getLstObjectDrawWithInputConnection();
		addTextDescriptor(NAME, indice++ + " - name");
		Ressources ressource = new Ressources();
		// ---
		String fullPath = Spagic1ToSpagic2Activator.getRessourcePath("/ressources/hl7Map.xml");
		String first = "";
		String[] comboDs = null;
		if (fullPath != null) {
			
			if (fullPath.substring(0, "file:".length()).compareTo("file:") == 0) {
				fullPath = fullPath.substring("file:".length());
			}
		
			ressource.setDocument(fullPath);
			
			
			NodeList nodeList = ressource.getXPathNodeSet("/TRANSFORMATIONS/TRANSFORMATION");
			Node node = null;
			NamedNodeMap attributes = null;
			
			comboDs = new String[nodeList.getLength()];
			for (int i = 0; i < nodeList.getLength(); i++) {
				node = nodeList.item(i);				
				attributes = node.getAttributes();
				comboDs[i] = (String)attributes.getNamedItem("name").getNodeValue();
			}
		}
		
		
		addComboBoxDescriptor(transformationName, indice++ +" - tranformationName", comboDs);
		// ---
		addComboBoxDescriptor(NAMEDEST, indice++
				+ " - select the destination service", lstStr);
	}

	/**
	 * return the id of this element.
	 * @return the id of this element.
	 */
	public final String getComponentId() {
		return ID;
	}
}
