/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.im.out.jbi.smx.utils;

import java.io.ByteArrayInputStream;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.wsdl.Definition;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.stp.im.ImPackage;
import org.eclipse.stp.im.Process;
import org.eclipse.stp.im.Property;
import org.eclipse.stp.im.ServiceBinding;
import org.eclipse.stp.im.Step;
import org.eclipse.stp.im.StpIntermediateModel;
import org.eclipse.stp.im.constants.ImConstants;
import org.eclipse.stp.im.runtime.IRuntime;
import org.eclipse.stp.im.runtime.ImRuntimeActivator;
import org.spagic.ui.helpers.MetaDBHelper;
import org.spagic.utils.ImPreprocessor;
import org.spagic.utils.SpagicJBIImPreprocessor;
import org.spagic.utils.SpagicUtils;
import org.spagic.utils.wsdl.WSDLUtils;

public class JBIJetUtils {
	
	public static final String MEP_IN_ONLY = "http://www.w3.org/2004/08/wsdl/in-only";
	public static final String MEP_IN_OUT = "http://www.w3.org/2004/08/wsdl/in-out";
	public static final String XML_HEADER = "<\\?xml .*\\?>(\\r\\n)*";
	
	public static final String PROPERTY_MAP = "property-map".intern();
	public static final String MAP_PROPERTY_MAP = "map-property-map".intern();
	public static final String BEAN_PROPERTY_MAP = "bean-property-map".intern();
	public static final String LIST_PROPERTY_MAP = "list-property-map".intern();
	
	private static List<String> folders = new ArrayList<String>();
	
	private static IRuntime runtime = null;
	private static Document jbiRuntimeSpecificDocument = null;	
	
	public static Map<String, String> serviceBindingsTemplateMap = new HashMap<String, String>();
	public static Map<String,Document> extensionsDocument = new HashMap<String,Document>();
	 
	public static final String SERVICE_UNIT_PROPERTY_NAME = "ServiceUnit";
	public static final String LW_SERVICE_UNIT ="LwContainerServiceUnit";
	public static final String REQUIRE_JNDI_DATASOURCE_PROPERTY_NAME ="RequireJndiDataSource";
	public static final String JNDI_DATASOURCE_PROPERTY_NAME ="JndiDataSource";
	
	
	public static String getRuntimeId(){
		return "runtime.jbi.smx";
	}
	
	
	public static void initJbiConfiguration() throws RuntimeException{
		try{
			ImPackage imPackage = ImPackage.eINSTANCE;
			runtime = ImRuntimeActivator.getRuntime(getRuntimeId());
			String specificRuntimeXMLString = runtime.getRuntimeSpecific();
			
			jbiRuntimeSpecificDocument = DocumentHelper.parseText(specificRuntimeXMLString);
			
			Map<String, String> extensionsRuntimesSpecific = runtime.getRuntimeSpecificForExtension();
			if ( extensionsRuntimesSpecific != null ){
				for (String key : extensionsRuntimesSpecific.keySet()){
					extensionsDocument.put(key, DocumentHelper.parseText(extensionsRuntimesSpecific.get(key)));
				}
			}
			folders.clear();
		}catch (Exception e) {
			throw new RuntimeException(e);
		}finally{
			
		}
	}
	
	//
	// UTILITIES METHOD
	//
	private static Node selectNode(Document document, String xPath ){
		return document.selectSingleNode(xPath);
	}
	
	private static Node selectNode(String xPath) {
		Node node = selectNode(jbiRuntimeSpecificDocument,xPath);

		if (node != null)
			return node;

		System.out.println("selectNode::xpath [" + xPath + "] evaluation on main rutime specific has no effect ");
		
		for (String extKey : extensionsDocument.keySet()) {
			System.out.println("selectNode::xpath [" + xPath + "] on runtime specific of extension [" + extKey + "]");
			node = selectNode(extensionsDocument.get(extKey), xPath);
			if (node != null)
				return node;
		}
		System.out.println(" selectNode::xpath::has null value on all runtime and extensions ");
		return null;
	}
	
	private static List<Node> selectNodes(Document document, String xPath ){
		return document.selectNodes(xPath);
	}
	
	private static List<Node> selectNodes(String xPath) {
		List<Node> nodes = selectNodes(jbiRuntimeSpecificDocument, xPath);

		if ((nodes != null) && (nodes.size() > 0)){
			return nodes;
		}


		System.out.println("selectNode::xpath [" + xPath + "] evaluation on main rutime specific has no effect ");
		
		for (String extKey : extensionsDocument.keySet()) {
			System.out.println("selectNode::xpath [" + xPath + "] on runtime specific of extension [" + extKey + "]");
			nodes = selectNodes(extensionsDocument.get(extKey), xPath);
			if ((nodes != null) && (nodes.size() > 0))
				return nodes;
		}
		
		return nodes;
	}
	
	
	
	public static String getServiceUnitNameForServiceBinding(ServiceBinding serviceBinding){
		
		String xPath = "/runtime-specific/service-binding-su-map/service-binding-su-map-entry[@serviceBindingName='"+serviceBinding.getName()+"']";
		Node node = selectNode(xPath);
		
		String suName = null;
		
		if (node != null){
			suName = node.valueOf("@serviceUnitName");
		}
		
		return suName;
		
	}
	public static Map<String, List<Step>> getServiceUnitListForProcessModels(org.eclipse.stp.im.Process aProcess){
		
		// 
		// To get the service unit for the step
		// me must get the right binding for the step and then see in the runtime specific configuration
		// which is the service unit for a particular binding
		//
		
		Map<String, List<Step>> suMap = new HashMap<String, List<Step>>();
		
		ServiceBinding serviceBindingForStepWithThisRuntime = null;
		String serviceUnitNameForBinding = null;
		
		for (Step step : aProcess.getSteps()){
			
			serviceBindingForStepWithThisRuntime = getRightStepBinding(step);
			
			if (serviceBindingForStepWithThisRuntime == null){
				throw new RuntimeException(" Not found a servicebing for step ["+step.getName()+"] for runtime ["+getRuntimeId()+"]");
			}
			
			serviceUnitNameForBinding = getServiceUnitNameForServiceBinding(serviceBindingForStepWithThisRuntime);
			
			
			if (serviceUnitNameForBinding == null){
				throw new RuntimeException(" Not found a service unit for service binding ["+serviceBindingForStepWithThisRuntime+"] for runtime ["+getRuntimeId()+"]");
			}
			
			if (suMap.containsKey(serviceUnitNameForBinding)){
					// Service Unit is already on su map
					// Add the step to su
					((List<Step>)suMap.get(serviceUnitNameForBinding)).add(step);
			}else{
					List<Step> stepsForSu = new ArrayList<Step>();
					stepsForSu.add(step);
					suMap.put(serviceUnitNameForBinding, stepsForSu);
			}
		}
		
		return suMap;
	}
	
	
	
	public static String getComponentNameForServiceUnit(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = selectNode(xPath);
		return node.valueOf("@componentName");
	}
	
	public static String getVersionAsStringForProcess(Process aProcess){
		Property processVersionProperty = aProcess.getProperties().get(ImConstants.PROCESS_VERSION);
		String version = null;
		if (processVersionProperty != null)
			version = (String)processVersionProperty.getValue();
		else
			version = "0";
		return version;
	}
	
	public static String getNamespaceURI(Process aProcess){
		return SpagicUtils.getNameSpaceForProcess(aProcess);
	}
	
	
	public static String getNamespacePrefix(Process aProcess){
		
		return SpagicUtils.getNameSpacePrefixForProcess(aProcess);
	}
	
	
	
	public static boolean isStandard(String suName){
	
		return !suName.equalsIgnoreCase("LwContainerServiceUnit");
	
	}
	
	
	
	public static String getRuntimeNameSpacePrefix() {
		
		return runtime.getNsprefix();
	
	}

	public static String getRuntimeNameSpaceURI() {
	
		return runtime.getNsuri();
	
	}
	
	public static String getJetTemplateForServiceBinding(ServiceBinding serviceBinding){

		String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
		Node node = selectNode(xPath);
		if (node == null)
			throw new RuntimeException(" Not found code-generator section for service binding ["+ serviceBinding.getName() + "]");
		return node.valueOf("@jetTemplate");
	
	}
	
	public static boolean isRequiredNamespaceMapForBinding(ServiceBinding serviceBinding){

		String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
		Node node = selectNode(xPath);
		if (node == null)
			return false;
		
		String requireNamespaceMap = node.valueOf("@requireNamespaceMap");
		if (requireNamespaceMap != null && isValid(requireNamespaceMap))
			return Boolean.valueOf(requireNamespaceMap);
		else	
			return false;
	
	}
	
	
	public static boolean injectDestinationAsExchangeTarget(ServiceBinding serviceBinding){
		String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
		Node node = selectNode(xPath);
		if (node == null)
			return false;
		
		String requireNamespaceMap = node.valueOf("@handleDestinationInjectingExchangeTarget");
		if (requireNamespaceMap != null && isValid(requireNamespaceMap))
			return Boolean.valueOf(requireNamespaceMap);
		else	
			return false;
	}
	
	public static String getLightweightClassForServiceBinding(ServiceBinding serviceBinding, Step s){
		if (serviceBinding.getName().equalsIgnoreCase("JBI-AdvancedJDBCService")){
			boolean isStored = Boolean.valueOf(JBIJetUtils.getRequiredPropertyValue(s, "FlagIsStoredProcdure"));
			return ( isStored ? "org.spagic.smx.components.jdbcstore.JDBCStoreProcedureComponent" : "org.spagic.smx.components.jdbcquery.JDBCAdvancedQueryComponent");
		}else{
			String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
			Node node = selectNode(xPath);
			return node.valueOf("@lwClassName");
		}
	}
	
	public static String getBeanClassForServiceBinding(ServiceBinding serviceBinding, Step s){
		if (serviceBinding.getName().equalsIgnoreCase("JBI-AdvancedJDBCService")){
			boolean isStored = Boolean.valueOf(JBIJetUtils.getRequiredPropertyValue(s, "FlagIsStoredProcdure"));
			return ( isStored ? "org.spagic.smx.components.jdbcstore.JDBCStoreProcedureComponent" : "org.spagic.smx.components.jdbcquery.JDBCAdvancedQueryComponent");
		}else{
			String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
			Node node = selectNode(xPath);
			return node.valueOf("@beanClassName");
		}
	}
	
	public static String getServiceUnitBriefName(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = selectNode(xPath);
		return node.valueOf("@briefName");
		
	}
	public static String getServiceUnitNameSpaceURI(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = selectNode(xPath);
		return node.valueOf("@nsuri");
	}
	
	public static String getServiceUnitNameSpacePrefix(String serviceUnitName){
		String xPath ="/runtime-specific/service-units/service-unit[@name='"+serviceUnitName+"']";
		Node node = selectNode(xPath);
		return node.valueOf("@nsprefix");
		
	}
	
	public static SimplePropertyMap parseSimplePropertyMap(Node n){
		String decorator = null;
		IDecorator decoratorInst = null;
		String produceConfiguration = null;

		String flagIsRef = null;
		String flagIsConst = null;
		String constValue = null;
		SimplePropertyMap aPropertyMap = null;
		if (n.getName().equalsIgnoreCase("constructor-arg-property-map")){
			 aPropertyMap = new ConstructorArgPropertyMap(n.valueOf("@im-model-name"), n
						.valueOf("@runtime-name"), Boolean.valueOf(n
						.valueOf("@required")));
		}else{
		   aPropertyMap = new SimplePropertyMap(n.valueOf("@im-model-name"), n
				.valueOf("@runtime-name"), Boolean.valueOf(n
				.valueOf("@required")));
		}
		decorator = n.valueOf("@decorator");
		if (isValid(decorator)) {
			try {
				decoratorInst = (IDecorator) Class.forName(decorator)
						.newInstance();
			} catch (Exception e) {
				e.printStackTrace();
				LogUtils.error(e);
			}
			aPropertyMap.setDecorator(decoratorInst);
		}

		String condition = n.valueOf("@condition");
		if (isValid(condition)) {
			aPropertyMap.setCondition(condition);
		}
		produceConfiguration = n.valueOf("@produceConfiguration");
		if (isValid(produceConfiguration)) {
			aPropertyMap.setRequired(Boolean.valueOf(produceConfiguration));
		}

		flagIsRef = n.valueOf("@isRef");
		if (isValid(flagIsRef)) {
			((SimplePropertyMap)aPropertyMap).setFlagRef(Boolean.valueOf(flagIsRef));
		}
		
		flagIsConst = n.valueOf("@is-const");
		if (isValid(flagIsConst)) {
			((SimplePropertyMap)aPropertyMap).setFlagConst(Boolean.valueOf(flagIsConst));
		}
		
		constValue = n.valueOf("@const-value");
		if (isValid(constValue)) {
			((SimplePropertyMap)aPropertyMap).setConstValue(constValue);
		}
		
		return aPropertyMap;
	}
	
	public static IPropertyMap parseMapPropertyMap(Node n){
		MapPropertyMap aPropertyMap = new MapPropertyMap(n.valueOf("@im-model-name"), n
			.valueOf("@runtime-name"), Boolean.valueOf(n
			.valueOf("@required")));
		
		
		String mapEntryKey = n.valueOf("@map-entry-key");
		String valueBeanClass = n.valueOf("@value-bean-class");
		aPropertyMap.setMapEntryKey(mapEntryKey);
		if (isValid(valueBeanClass)){
			aPropertyMap.setValueBeanClass(valueBeanClass);
		}
		List<Node> beanMappingsProperties = n.selectNodes("value-bean-property");
		List<SimplePropertyMap>  beanMappingPropertyMap = new ArrayList<SimplePropertyMap>();
		for (Node beanMappingNode : beanMappingsProperties){
			beanMappingPropertyMap.add(parseSimplePropertyMap(beanMappingNode));
		}
		aPropertyMap.setBeanMappingProperty(beanMappingPropertyMap);
		
		return aPropertyMap;
	}
	
	public static IPropertyMap parseListPropertyMap(Node n){
		ListPropertyMap aPropertyMap = new ListPropertyMap(n.valueOf("@im-model-name"), n
			.valueOf("@runtime-name"), Boolean.valueOf(n
			.valueOf("@required")));
		
	
		String valueBeanClass = n.valueOf("@value-bean-class");
		
		if (isValid(valueBeanClass)){
			aPropertyMap.setValueBeanClass(valueBeanClass);
		}
		List<Node> beanMappingsProperties = n.selectNodes("value-bean-property");
		List<SimplePropertyMap>  beanMappingPropertyMap = new ArrayList<SimplePropertyMap>();
		for (Node beanMappingNode : beanMappingsProperties){
			beanMappingPropertyMap.add(parseSimplePropertyMap(beanMappingNode));
		}
		aPropertyMap.setBeanMappingProperty(beanMappingPropertyMap);
		return aPropertyMap;
	}
	
	public static IPropertyMap getPropertyMapFromNode(Node n) {
		IPropertyMap aPropertyMap = null;
		
		if (n.getName().equalsIgnoreCase(PROPERTY_MAP) || n.getName().equalsIgnoreCase("constructor-arg-property-map")) {
			return parseSimplePropertyMap(n);
		}else if (n.getName().equalsIgnoreCase(MAP_PROPERTY_MAP)) {
			return parseMapPropertyMap(n);
		}else if (n.getName().equalsIgnoreCase(LIST_PROPERTY_MAP)) {
			return parseListPropertyMap(n);
		}else if (n.getName().equalsIgnoreCase(BEAN_PROPERTY_MAP)) {
			aPropertyMap = new BeanPropertyMap(n.valueOf("@im-model-name"), n
					.valueOf("@runtime-name"), Boolean.valueOf(n
					.valueOf("@required")));
			String beanClass = n.valueOf("@bean-class");
			((BeanPropertyMap)aPropertyMap).setBeanClassName(beanClass);
			List<Node> children = new ArrayList<Node>();
			List<IPropertyMap> childrenPropertyMap = new ArrayList<IPropertyMap>();
			children.addAll(n.selectNodes(PROPERTY_MAP));
			children.addAll(n.selectNodes(MAP_PROPERTY_MAP));
			children.addAll(n.selectNodes(BEAN_PROPERTY_MAP));
			children.addAll(n.selectNodes(LIST_PROPERTY_MAP));
			children.addAll(n.selectNodes("constructor-arg-property-map"));
			for (Node n1 : children){
				childrenPropertyMap.add(getPropertyMapFromNode(n1));
			}
			((BeanPropertyMap)aPropertyMap).setChildrenProperties(childrenPropertyMap);
		}
		return aPropertyMap;
	}
	
	public static List<IPropertyMap> getCodeGeneratorPropertyMap(ServiceBinding serviceBinding){
		
		// First off all i must see which is the document containining the
		String xPathServiceBinding  ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']";
		
		Document targetDocument = null;
		if (selectNode(jbiRuntimeSpecificDocument, xPathServiceBinding) != null)
			targetDocument = jbiRuntimeSpecificDocument;
		
		for (String extKey : extensionsDocument.keySet()) {
			Document tDoc = extensionsDocument.get(extKey);
			if (selectNode(tDoc, xPathServiceBinding) != null)
				targetDocument = tDoc;
		}
		
		List<IPropertyMap>  propertiesMap = new ArrayList<IPropertyMap>();
		
		if (targetDocument != null){
			
			String xPath1 = xPathServiceBinding + "/" + PROPERTY_MAP;
			String xPath2 = xPathServiceBinding + "/" + MAP_PROPERTY_MAP;
			String xPath3 = xPathServiceBinding + "/" + BEAN_PROPERTY_MAP;
			String xPath4 = xPathServiceBinding + "/" + LIST_PROPERTY_MAP;
			List<Node> nodes = new ArrayList<Node>();
			nodes.addAll(targetDocument.selectNodes(xPath1));
			nodes.addAll(targetDocument.selectNodes(xPath2));
			nodes.addAll(targetDocument.selectNodes(xPath3));
			nodes.addAll(targetDocument.selectNodes(xPath4));
			
			for (Node n : nodes ){
				propertiesMap.add(getPropertyMapFromNode(n));
			}
		}
		
		return propertiesMap;
	
	}
	
	public static List<CopyResource> getCodeGeneratorCopyResources(ServiceBinding serviceBinding){
		
		String xPath ="/runtime-specific/code-generators/code-generator[@serviceBindingName='"+serviceBinding.getName()+"']/copy-resource";	
		List<Node> nodes = selectNodes(xPath);
		
		CopyResource aCopyResource = null;
		List<CopyResource>  copyResources = new ArrayList<CopyResource>();
		
		String identifyProperty = null;
		String produceConfiguration = null;
		for (Node n : nodes ){
			identifyProperty = n.valueOf("@identifyingProperty");
			aCopyResource = new CopyResource(identifyProperty);
			
			copyResources.add(aCopyResource);
		}
		
		return copyResources;
	}

	public static boolean isValid(String s) {
		return (s != null && s.trim().length() > 0);
	}

	public static String getRequiredPropertyValue(Step step, String imName) {
	
		Property aProperty = step.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : null;
		
		if (!JBIJetUtils.isValid(valueInIm)){
			throw new RuntimeException(" Property ["+imName+"] --> is required to generate code for Step ["+step.getName()+"] with Binding ["+JBIJetUtils.getRightStepBinding(step).getName()+"]");
		}
		return valueInIm;
	}

	public static String getOptionalPropertyValue(Step step,
			String imName, String defaultValue) {
		
		Property aProperty = step.getProperties().get(imName);
		String valueInIm = aProperty != null ? aProperty.getValue().toString() : defaultValue;
		
		return valueInIm;
	}

	
	
	/**
	 * 
	 * @param s
	 * @return the right service binding for the runtime we're using during the transformation
	 */
	public static ServiceBinding getRightStepBinding(Step s){
		
		String runtimeID = getRuntimeId();
		Property runtimPropertyForServiceBinding = null;
		for (ServiceBinding sb : s.getStepbindings()){
			runtimPropertyForServiceBinding = sb.getProperties().get(ImConstants.IM_STEP_BINDING_RUNTIME_ID);
			if (runtimPropertyForServiceBinding != null){
				if (runtimPropertyForServiceBinding.getValue().equals(runtimeID)){
					return sb;
				}
			}else{
				LogUtils.warn("ServiceBinding["+sb.getName()+"] -> for Step["+s.getName()+"] does not have a property called "+ ImConstants.IM_POOL_RUNTIME_ID);
			}
		}
		LogUtils.warn("Not found for Step["+s.getName()+"] a binding for runtime "+ ImConstants.IM_POOL_RUNTIME_ID);
		return null;
		
	}
	
	public static boolean isRequiredNamespaceContext(String suName){
		if (suName.equalsIgnoreCase("DroolsServiceUnit") ||
			suName.equalsIgnoreCase("EIPServiceUnit") ||
			suName.equalsIgnoreCase("RulesServiceUnit")) {
			return true;
		}
		return false;
	}
	
	public static void addFolder(String folderName){
		folders.add(folderName);
	}
	


	public static List<String> getFolders() {
		return folders;
	}


	public static void setFolders(List<String> folders) {
		JBIJetUtils.folders = folders;
	}
	
	public static String getRelativeFileName(String original){
		String relPath = original;
		if (original.indexOf("/") > 0)
			relPath = relPath.substring(original.lastIndexOf("/") + 1);
		if (original.indexOf("\\") > 0)
			relPath = relPath.substring(original.lastIndexOf("\\") + 1);
		return relPath;
	}
	
	public static List<String> getJndiDatasourceNames(List<Step> stepList){
		List<String> datasources = new ArrayList<String>();
		String flgRequireDataSource = null;
		String dsName = null;
		for (Step aStep : stepList){
			flgRequireDataSource = getOptionalPropertyValue(aStep, REQUIRE_JNDI_DATASOURCE_PROPERTY_NAME, "false");
			if (flgRequireDataSource.equalsIgnoreCase("true")){
				dsName = getRequiredPropertyValue(aStep, JNDI_DATASOURCE_PROPERTY_NAME);
				if (!datasources.contains(dsName))
					datasources.add(dsName);
			}
		}
		return datasources;
	}
	
	public static Map<String, String> getNamespaceMap(){
		return MetaDBHelper.getNameSpaceMap();		
	}
	
	public static String getObservableAttributeXML(List<org.eclipse.stp.im.Process> processList) {
		return MetaDBHelper.getObservableAttributeXML(processList);
	}
	
	public static void jbiIntermediateModelPreprocessor(
			StpIntermediateModel aStpIntermediateModel) {

		ImPreprocessor imPreprocessor = new SpagicJBIImPreprocessor(getRuntimeId());
		imPreprocessor.process(aStpIntermediateModel);		
	
	}
	
	
	// ----- SPAGIC STP IM UTILS
	
	
	
	public static boolean existFileInProject(String projectName, String wsdlFile){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject proj = root.getProject(projectName);
		IFile file = proj.getFile(wsdlFile);
		return file.exists();
	}
	
	public static String fixWSDL(Process process, 
								Step sourceStep, 
								Step destinationStep, 
								String projectName, 
								String wsdlFileRelToProject, 
								String locationURI,
								String serviceBindingName,
								boolean isTransformerPipeline){
		IWorkspace workspace = ResourcesPlugin.getWorkspace();
		IWorkspaceRoot root = workspace.getRoot();
		IProject proj = root.getProject(projectName);
		IFile file = proj.getFile(wsdlFileRelToProject);
		if (!file.exists()){
			throw new RuntimeException(" Wsdl File "+ wsdlFileRelToProject + " Not exist in project " + projectName+" ");
		}
		try{
			String destinationWsdlFileName = "WsdlFiles/"+SpagicUtils.getStepLocalNamePart(process, sourceStep) + ".wsdl";
			
			IFile destinationWsdlFile = proj.getFile(destinationWsdlFileName);
		
			Definition def = WSDLUtils.readWSDLDefinition(file.getLocationURI());
			
			def = WSDLUtils.flatDefinition(def);
			String style = WSDLUtils.IN_OUT_STYLE;
			if (   (serviceBindingName.equalsIgnoreCase("JBI-CXF-InputBindingComponent"))
				|| (serviceBindingName.equalsIgnoreCase("JBI-CXF-OutputBindingComponent"))) {
				style = WSDLUtils.IN_ONLY_STYLE;
			}
			
			String portTypeName = SpagicUtils.getStepLocalNamePart(process, sourceStep);
			String serviceName = null;
			
			if ((serviceBindingName.equalsIgnoreCase("JBI-CXF-OutputBindingComponent"))){
				serviceName = portTypeName;
			} else if ( serviceBindingName.equalsIgnoreCase("JBI-CXF-InputOutputBindingComponent")
					&& isTransformerPipeline){
				serviceName = portTypeName;
			} else {
				serviceName = SpagicUtils.getStepLocalNamePart(process, destinationStep);				
			}
			
			
			
			def = WSDLUtils.getJBICompliantDefinition(def, 
													  SpagicUtils.getNameSpacePrefixForProcess(process), 
													  SpagicUtils.getNameSpaceForProcess(process), 
													  portTypeName,
													  serviceName,
													  locationURI,
													  style);
			
			saveDefininitionOnIFile(def, destinationWsdlFile);
			return destinationWsdlFileName;
		}catch (Exception e) {
			throw new RuntimeException(e);
		}

	}
	
	protected static void saveDefininitionOnIFile(Definition def, IFile file){
		
		
		
		try{
			Map<String, String> ns = def.getNamespaces();
			for ( String prefix : ns.keySet()){
				System.out.println("["+prefix+"] -> ["+ns.get(prefix)+"]");
			}
			StringWriter sw = new StringWriter();
			WSDLUtils.writeWSDL(def, sw);
			byte[] bytes = sw.toString().getBytes();
			ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
			if (file.exists()){
				file.setContents(bais, IFile.FORCE, null);
			}else{
				file.create(bais, true, new NullProgressMonitor());
			}

		}catch (Exception e) {
			throw new RuntimeException(e);
		}
		
	}
	
	/**
	 * Removes the xml header.
	 * 
	 * @param xmlString the xml string
	 * 
	 * @return the string without the XML header
	 */
	public static String removeXMLHeader(String xmlString) {
		return xmlString.replaceFirst(XML_HEADER, "");
	}
	
	public static String escapeForXML(String s){
		String result = s.replaceAll("<", "&lt;");
		return result;
	}

	
}
