/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.ui.dialog;

import java.io.FileWriter;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.PlatformUI;
import org.spagic.ui.SpagicUIActivator;
import org.spagic.ui.bean.ServiceRegistryConfiguration;
import org.spagic.utils.SpagicLogger;

public class ServiceRegistryDialog extends SpagicTitleAreaDialog {

	private ServiceRegistryConfiguration serviceRegistry = null;
	private Text inquiryT = null;
	private Text publishT = null;
	private Text userT = null;
	private Text pswT = null;
	private Text defaultOrganizationT = null;
	
	public ServiceRegistryDialog(Shell parentShell) {
		super(parentShell);
	}

	public ServiceRegistryDialog(Shell parentShell, ServiceRegistryConfiguration sr) {
		super(parentShell);
		this.serviceRegistry = sr;
	}
	/**
	 * @see org.eclipse.jface.window.Window#create() We complete the dialog with
	 *      a title and a message
	 */
	public void create() {
		super.create();
		setTitle("Update Service Registry");
		setMessage("Update Service Registry");
	}

	@Override
	protected Control createContents(Composite parent) {
		Control ctrl = super.createContents(parent);
		return ctrl;
	}

	protected Control createDialogArea(Composite parent) {
		Composite aComposite = new Composite(parent, SWT.NONE);
		final GridLayout layout = new GridLayout();
		layout.marginWidth = 15;
		layout.marginHeight = 10;
		layout.numColumns = 5;
		aComposite.setLayout(layout);
		aComposite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;

		// Name
		CLabel nameL = new CLabel(aComposite, SWT.RIGHT);
		nameL.setText(" Name ");
		Text nameT = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		nameT.setLayoutData(data);
		
		// inquiry URL
		CLabel inquiryL = new CLabel(aComposite, SWT.RIGHT);
		inquiryL.setText(" Inquiry URL ");
		inquiryT = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		inquiryT.setLayoutData(data);
		
		// publish URL
		CLabel publishL = new CLabel(aComposite, SWT.RIGHT);
		publishL.setText(" Publish URL ");
		publishT = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		publishT.setLayoutData(data);
		
		// user
		CLabel userL = new CLabel(aComposite, SWT.RIGHT);
		userL.setText(" User ");
		userT = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		userT.setLayoutData(data);
		
		// password
		CLabel pswL = new CLabel(aComposite, SWT.RIGHT);
		pswL.setText(" Password ");
		pswT = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		pswT.setLayoutData(data);
		
		//defaultOrganization
		CLabel defaultOrganizationL = new CLabel(aComposite, SWT.RIGHT);
		defaultOrganizationL.setText(" Default Organization ");
		defaultOrganizationT = new Text(aComposite, SWT.BORDER | SWT.FLAT);
		data = new GridData(GridData.FILL_HORIZONTAL);
		data.widthHint = 200;
		data.horizontalSpan = 4;
		defaultOrganizationT.setLayoutData(data);
		
		if (this.serviceRegistry != null){
			nameT.setText(serviceRegistry.getName());
			inquiryT.setText(serviceRegistry.getInquiryUrl());
			publishT.setText(serviceRegistry.getPublishUrl());
			userT.setText(serviceRegistry.getUser());
			pswT.setText(serviceRegistry.getPsw());
			defaultOrganizationT.setText(serviceRegistry.getDefaultOrganitation());
			nameT.setEditable(false);
		}
		return parent;
	}

	protected void okPressed() {
		if (serviceRegistry != null)
			modifyServiceRegistry();
	}
	
	protected void modifyServiceRegistry() {
		String name = serviceRegistry.getName();
		SAXReader reader = new SAXReader(false);
		try {
			Document serviceRegistryDocument = reader.read(SpagicUIActivator.getServiceRegistryFile());
			List<org.dom4j.Node> nodeList = serviceRegistryDocument.selectNodes("/SERVICE_REGISTRIES/SERVICE_REGISTRY");
			Document document = DocumentHelper.createDocument();
	        Element rootServiceRegisty = document.addElement( "SERVICE_REGISTRIES" );

			for (org.dom4j.Node n : nodeList){
				if (!((String)n.valueOf("@name")).equalsIgnoreCase(name)){
					rootServiceRegisty.addElement( "SERVICE_REGISTRY" )
			        .addAttribute( "name", n.valueOf("@name"))
			        .addAttribute( "inquiryUrl", n.valueOf("@inquiryUrl")) 
			        .addAttribute( "publishUrl", n.valueOf("@publishUrl" ))
			        .addAttribute( "user", n.valueOf("@user")) 
			        .addAttribute( "password", n.valueOf("@password")) 
			        .addAttribute( "defaultOrganitation", n.valueOf("@defaultOrganitation"));
				}else{
					rootServiceRegisty.addElement( "SERVICE_REGISTRY" )
			        .addAttribute( "name", n.valueOf("@name"))
			        .addAttribute( "inquiryUrl", inquiryT.getText()) 
			        .addAttribute( "publishUrl", publishT.getText())
			        .addAttribute( "user", userT.getText()) 
			        .addAttribute( "password", pswT.getText()) 
			        .addAttribute( "defaultOrganitation", defaultOrganizationT.getText());
				}
				
			}
			FileWriter fileServiceRegistry = new FileWriter(SpagicUIActivator.getSpagicStudioConfigurationPath() + "/serviceRegistry.xml");
			document.write(fileServiceRegistry);
			fileServiceRegistry.flush();
			fileServiceRegistry.close();
			close();
			SpagicUIActivator.getDefault().initServiceRegistry();
		} catch (Exception e) {
			SpagicLogger.error(SpagicUIActivator.PLUGIN_ID, e.getMessage(), e);
			MessageDialog.openError(PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getShell(), "Spagic",
					"An error is occurred [" + e.getMessage() + "]");

		}
	}

	@Override
	protected void buttonPressed(int buttonId) {

		super.buttonPressed(buttonId);
	}

}
