/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.utils.preference;

import java.util.List;

import org.dom4j.Document;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.ComboFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.spagic.utils.SpagicUtilsActivator;

/**
 * This class represents a preference page that
 * is contributed to the Preferences dialog. By 
 * subclassing <samp>FieldEditorPreferencePage</samp>, we
 * can use the field support built into JFace that allows
 * us to create a page that is small and knows how to 
 * save, restore and apply itself.
 * <p>
 * This page is used to modify preferences only. They
 * are stored in the preference store that belongs to
 * the main plug-in class. That way, preferences can
 * be accessed directly via the preference store.
 */

public class SpagicPreferencePage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage {

	public SpagicPreferencePage() {
		
		super(GRID);
		setPreferenceStore(SpagicUtilsActivator.getDefault().getPreferenceStore());
		setDescription("Preferences For Spagic Studio Plugin");
	
	}
	
	/**
	 * Creates the field editors. Field editors are abstractions of
	 * the common GUI blocks needed to manipulate various types
	 * of preferences. Each field editor knows how to save and
	 * restore itself.
	 */
	public void createFieldEditors() {
		GridLayout layoutPreference = new GridLayout();
		layoutPreference.numColumns = 1;
		Composite aComposite = new Composite(getFieldEditorParent(), SWT.NONE);
		aComposite.setLayout(layoutPreference);
		
		Group serviceManagerGroup = new Group(aComposite,SWT.SHADOW_ETCHED_IN);
		serviceManagerGroup.setText("Service Manager");
		serviceManagerGroup.setSize(350, 100);
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_BASE_NAME_SPACE, "Spagic Base Name Space Prefix", serviceManagerGroup));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_SERVICE_MANAGER_HOST, "Spagic Service Manager Host", serviceManagerGroup));
		
		Group metaDBGroup = new Group(aComposite,SWT.SHADOW_ETCHED_IN);
		metaDBGroup.setText("Meta DB");
		metaDBGroup.setSize(350, 300);
		addField(
			new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_URL, "Spagic DB URL", metaDBGroup));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_DRIVER, "Spagic DB DRIVER", metaDBGroup));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_USER, "Spagic DB USER", metaDBGroup));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_PASSWORD, "Spagic DB PASSWORD", metaDBGroup));
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SPAGIC_DB_HIB_DIALECT, "Spagic DB DIALECT", metaDBGroup));
		addField(
				new BooleanFieldEditor(
					SpagicPreferenceConstants.P_SPAGIC_DB_SHOW_SQL,
					"Show sql",
					metaDBGroup));
		
		Group serviceRegistryGroup = new Group(aComposite,SWT.SHADOW_ETCHED_IN);
		serviceRegistryGroup.setText("Service Registry");
		serviceManagerGroup.setSize(350, 50);
		String[][] serviceRegistryList = new String[2][2];
		serviceRegistryList[0][0] = "uddi";
		serviceRegistryList[0][1] = "uddi";
		serviceRegistryList[1][0] = "ebXML";
		serviceRegistryList[1][1] = "ebXML";
		
		addField(
				new ComboFieldEditor(SpagicPreferenceConstants.P_SERVICE_REGISTRY, "Service Registry", serviceRegistryList, serviceRegistryGroup));		
		
		Group graphVizGroup = new Group(aComposite,SWT.SHADOW_ETCHED_IN);
		graphVizGroup.setText("Graph Viz");
		graphVizGroup.setSize(350, 150);
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_GRAPH_VIZ_DOT_EXE, "Graphviz dot program Location",graphVizGroup));
		
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_GRAPH_VIZ_TEMP_FOLDER, "Graphviz temp folder", graphVizGroup));
		
		Group otherGroup = new Group(aComposite,SWT.SHADOW_ETCHED_IN);
		otherGroup.setText("GetResources Service");
		addField(
				new StringFieldEditor(SpagicPreferenceConstants.P_SMX_GET_RESOURCES, "SMX GetResources Service ", otherGroup));
		
		
	}

	/** (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		
	}

	@Override
	protected void contributeButtons(Composite parent) {
		super.contributeButtons(parent);
	}
}